﻿//MainDialog.cpp
//メインダイアログ

//`~^`~^`~^`~^`~^`~^`~^`~^`~^`~^`~^`~^`~^`~^`~^`~^`~^`~^`
//          TimeStamp Keeper Ver.1.00 by x@rgs
//              under NYSL Version 0.9982
//
//`~^`~^`~^`~^`~^`~^`~^`~^`~^`~^`~^`~^`~^`~^`~^`~^`~^`~^`


#include"StdAfx.h"
#include"MainDialog.h"

using namespace sslib;


namespace{
	class DeferPos{
		public:
		DeferPos(int num=1):m_hdwp(NULL){m_hdwp=::BeginDeferWindowPos(num);}
		virtual ~DeferPos(){::EndDeferWindowPos(m_hdwp);}
		private:
			HDWP m_hdwp;
		public:
			bool move(HWND wnd,HWND insert_after,int x,int y,int cx,int cy,UINT flags){
				return NULL!=(m_hdwp=::DeferWindowPos(m_hdwp,wnd,insert_after,x,y,cx,cy,flags));
			}
	};

	//Milliseconds対応
	void longlong2SYSTEMTIME(SYSTEMTIME* result_st,const long long date_time){
		result_st->wYear=static_cast<WORD>(date_time/10000000000000);
		result_st->wMonth=static_cast<WORD>((date_time/100000000000)%100);
		result_st->wDay=static_cast<WORD>((date_time/1000000000)%100);
		result_st->wHour=static_cast<WORD>((date_time/10000000)%100);
		result_st->wMinute=static_cast<WORD>((date_time/100000)%100);
		result_st->wSecond=static_cast<WORD>((date_time/1000)%100);
		result_st->wMilliseconds=static_cast<WORD>(date_time%1000);
	}

	void getTimeStampStr(TCHAR* result,const TCHAR* file_path){
		FILETIME ft,fttmp;
		SYSTEMTIME st;
		File file(file_path,
				  OPEN_EXISTING,
				  GENERIC_READ,
				  FILE_SHARE_READ|FILE_SHARE_WRITE);

		file.getFileTime(&ft);
		::FileTimeToLocalFileTime(&ft,&fttmp);
		::FileTimeToSystemTime(&fttmp,&st);

		lstrcpy(result,format(_T("%04d/%02d/%02d %02d:%02d:%02d.%03d"),
							  st.wYear,st.wMonth,st.wDay,st.wHour,st.wMinute,st.wSecond,st.wMilliseconds).c_str());
		return;
	}

	bool setTimeStamp(const TCHAR* file_path,FILETIME& ft){
		DWORD attr=::GetFileAttributes(file_path);

		::SetFileAttributes(file_path,
							attr&~FILE_ATTRIBUTE_READONLY);

		File file(file_path,
				  OPEN_EXISTING,
				  GENERIC_WRITE,
				  FILE_SHARE_READ);

		bool result=file.setFileTime(&ft);

		::SetFileAttributes(file_path,attr);

		return result;
	}

	void addTimeStampItem(ListView* listview,const TCHAR* file_path,const int index){
		LV_ITEM lvitem={};
		std::vector<TCHAR> timestamp(128);

		getTimeStampStr(&timestamp[0],file_path);

		lvitem.mask=LVIF_TEXT;
		lvitem.pszText=&timestamp[0];
		lvitem.iItem=index;
		lvitem.iSubItem=1;
		listview->setItem(&lvitem);
	}
}

INT_PTR MainDialog::onInitDialog(WPARAM wparam,LPARAM lparam){
	//アイコンの設定(タイトルバー)
	setIcon(IDI_ICON1);

	//タイトルの設定(タイトルバー)
	::SetWindowText(handle(),(tstring(_T("TimeStamp Keeper Ver."))+SOFTWARE_VERSION).c_str());

	m_listview=new FileListView(handle());
	if(m_listview==NULL)return false;

	RECT rc={0};

	::GetWindowRect(handle(),&rc);

	//onGetMinMaxInfo()用
	m_wnd_width=rc.right-rc.left;
	m_wnd_height=rc.bottom-rc.top;

	::GetWindowRect(m_listview->handle(),&rc);

	m_listview->insertColumn(0,
							 _T("ファイル名"),
							 LVCFMT_LEFT,
							 rc.right-rc.left);
	m_listview->insertColumn(1,
							 _T("更新日時"),
							 LVCFMT_LEFT,
							 140);

	m_listview->setExtendedListViewStyle(m_listview->getExtendedListViewStyle()|
										 //LVS_EX_FULLROWSELECT(行全体選択)
										 LVS_EX_FULLROWSELECT|
										 //LVS_EX_GRIDLINES(罫線表示)
										 LVS_EX_GRIDLINES|
										 //LVS_EX_CHECKBOXES(チェックボックス表示)
										 LVS_EX_CHECKBOXES);

	//サイズ変更時に場所を変更
	m_wnd_size_info=new SIZE_INFO(handle(),getDlgItem(IDC_LIST1));

	{
		//引数解析
		std::vector<tstring>& filepaths=static_cast<CommandArgument*>(param())->filepaths();

		for(std::vector<tstring>::size_type i=0,size=filepaths.size();i<size;++i){
			int item=m_listview->insertItem(filepaths[i].c_str());
			if(item!=-1){
				addTimeStampItem(m_listview,filepaths[i].c_str(),item);
				m_listview->setCheckState(item);
				m_listview->setItemData(FileListView::STATE_INIT,item);
			}
		}
	}

	::DragAcceptFiles(handle(),true);

	return true;
}

INT_PTR MainDialog::onCommand(WPARAM wparam,LPARAM lparam){
	switch(LOWORD(wparam)){
		case IDM_MENU_RESTORE:
			sendMessage(WM_COMMAND,IDOK,0);
			break;
		case IDM_MENU_CLEAR:
			m_listview->clear();
			break;
		default:
			break;
	}
	return false;
}

INT_PTR MainDialog::onNotify(WPARAM wparam,LPARAM lparam){
	switch(((LPNMHDR)lparam)->idFrom){
		case 0:
			if(((LPNMHDR)lparam)->hwndFrom==::GetDlgItem(m_listview->handle(),0)){
				switch(((LPNMHDR)lparam)->code){
					case NM_CLICK:
					case NM_RCLICK:
						::SetFocus(m_listview->handle());
						return true;
					default:
						break;
				}
			}
			break;
		case IDC_LIST1:
			//子へ投げる
			SendMessage(m_listview->handle(),WM_NOTIFY,wparam,lparam);
			return ::GetWindowLong(handle(),DWL_MSGRESULT);
		default:
			break;
	}//switch(((LPNMHDR)lparam)->idFrom)
	return false;
}

INT_PTR MainDialog::onOk(){
	restoreTimeStamp();
	return false;
}

INT_PTR MainDialog::onDropFiles(HDROP drop_handle){
	DropFiles drop_files(drop_handle);

	for(size_t i=0,drop_files_count=drop_files.getCount(),count=m_listview->getItemCount();i<drop_files_count;i++){
		int item=m_listview->insertItem(drop_files.getFile(i).c_str(),count+i);

		if(item!=-1){
			addTimeStampItem(m_listview,drop_files.getFile(i).c_str(),item);
			m_listview->setCheckState(item);
			m_listview->setItemData(FileListView::STATE_INIT,item);
		}
	}
	return true;
}

INT_PTR MainDialog::onSize(WPARAM wparam,LPARAM lparam){
	if(!m_wnd_size_info)return false;

	DeferPos defer_pos;

	defer_pos.move(m_wnd_size_info->wnd,
				   NULL,
				   0,0,
				   LOWORD(lparam)-m_wnd_size_info->width_diff,
				   HIWORD(lparam)-m_wnd_size_info->height_diff,
				   SWP_NOMOVE|SWP_NOZORDER);
	return false;
}

INT_PTR MainDialog::onCopyData(WPARAM wparam,LPARAM lparam){
	PCOPYDATASTRUCT copy_data=reinterpret_cast<PCOPYDATASTRUCT>(lparam);

	if(copy_data->dwData==1){
		int item=m_listview->insertItem(static_cast<LPTSTR>(copy_data->lpData));
		if(item!=-1){
			addTimeStampItem(m_listview,static_cast<LPTSTR>(copy_data->lpData),item);
			m_listview->setCheckState(item);
			m_listview->setItemData(FileListView::STATE_INIT,item);
		}
		return true;
	}
	return false;
}

INT_PTR MainDialog::onGetMinMaxInfo(WPARAM wparam,LPARAM lparam){
	LPMINMAXINFO info=reinterpret_cast<LPMINMAXINFO>(lparam);

	info->ptMinTrackSize.x=m_wnd_width;
	info->ptMinTrackSize.y=m_wnd_height;
	return true;
}

INT_PTR MainDialog::onMessage(UINT message,WPARAM wparam,LPARAM lparam){
	switch(message){
		case WM_COPYDATA:
			return onCopyData(wparam,lparam);
		case WM_GETMINMAXINFO:
			return onGetMinMaxInfo(wparam,lparam);
		case WM_SETFOCUS:
			::SetFocus(m_listview->handle());
			return true;
		default:
			break;
	}
	return false;
}

void MainDialog::restoreTimeStamp(){
	int cnt=m_listview->getItemCount();
	if(!cnt)return;

	for(int item=0;item<cnt;++item){
		if(m_listview->getCheckState(item)){
			tstring timestamp_str=m_listview->getItemText(item,1);

			if(!path::fileExists(m_listview->getItemText(item).c_str())){
				m_listview->setItemData(FileListView::STATE_ERROR,item);
				::MessageBox(handle(),format(_T("ファイルが見つかりません。\n%s"),
											 m_listview->getItemText(item).c_str()).c_str(),
							 (tstring(_T("TimeStamp Keeper Ver."))+SOFTWARE_VERSION).c_str(),MB_ICONSTOP);
				continue;
			}

			struct isndigit{bool operator()(char c)const{return !isdigit(c);}};
			timestamp_str.erase(std::remove_if(timestamp_str.begin(),timestamp_str.end(),isndigit()),timestamp_str.end());

			long long ll=_tcstoll(timestamp_str.c_str(),NULL,10);
			SYSTEMTIME st;
			FILETIME ft,fttmp;

			longlong2SYSTEMTIME(&st,ll);
			::SystemTimeToFileTime(&st,&fttmp);
			::LocalFileTimeToFileTime(&fttmp,&ft);

			if(setTimeStamp(m_listview->getItemText(item).c_str(),ft)){
				m_listview->setItemData(FileListView::STATE_COMPLETE,item);
			}else{
				m_listview->setItemData(FileListView::STATE_ERROR,item);
				::MessageBox(handle(),format(_T("タイムスタンプの復元に失敗しました。\n%s [%s]"),
											 m_listview->getItemText(item).c_str(),
											 timestamp_str.c_str()
											 ).c_str(),
							 (tstring(_T("TimeStamp Keeper Ver."))+SOFTWARE_VERSION).c_str(),MB_ICONSTOP);
			}
		}
	}
}
